@@ -7,6 +7,10 @@ visualizations using Leaflet and other web technologies.
 from pathlib import Path
 from typing import List, Tuple, Optional, Callable
 import json
+import logging
+
+# Module-level logger
+logger = logging.getLogger(__name__)
 
 
 def prepare_mosaic_for_web(
@@ -16,13 +20,13 @@ def prepare_mosaic_for_web(
     progress_callback: Optional[Callable] = None,
 ) -> str:
     """Prepare an RGB mosaic for web visualization by reprojecting if needed.
-    
+
     Args:
         input_mosaic: Path to input RGB mosaic (3-band GeoTIFF)
         output_path: Output path for web-ready mosaic
         target_crs: Target CRS for web visualization (default: Web Mercator)
         progress_callback: Optional progress callback
-        
+
     Returns:
         Path to web-ready mosaic (may be same as input if no reprojection needed)
     """
@@ -31,40 +35,44 @@ def prepare_mosaic_for_web(
         from rasterio.warp import reproject, calculate_default_transform, Resampling
     except ImportError:
         raise ImportError("rasterio required: pip install rasterio")
-    
+
     if progress_callback:
         progress_callback(10, 100, "Checking mosaic CRS...")
-    
+
     # Check if reprojection is needed
     with rasterio.open(input_mosaic) as src:
         if str(src.crs) == target_crs:
             if progress_callback:
                 progress_callback(100, 100, f"Mosaic already in {target_crs}")
             return input_mosaic
-        
+
         if progress_callback:
-            progress_callback(20, 100, f"Reprojecting from {src.crs} to {target_crs}...")
-        
+            progress_callback(
+                20, 100, f"Reprojecting from {src.crs} to {target_crs}..."
+            )
+
         # Calculate transform and dimensions for target CRS
         dst_transform, dst_width, dst_height = calculate_default_transform(
             src.crs, target_crs, src.width, src.height, *src.bounds
         )
-        
+
         # Create output profile
         profile = src.profile.copy()
-        profile.update({
-            'crs': target_crs,
-            'transform': dst_transform,
-            'width': dst_width,
-            'height': dst_height,
-            'compress': 'lzw',
-        })
-        
+        profile.update(
+            {
+                "crs": target_crs,
+                "transform": dst_transform,
+                "width": dst_width,
+                "height": dst_height,
+                "compress": "lzw",
+            }
+        )
+
         if progress_callback:
             progress_callback(50, 100, "Writing reprojected mosaic...")
-        
+
         # Write reprojected mosaic
-        with rasterio.open(output_path, 'w', **profile) as dst:
+        with rasterio.open(output_path, "w", **profile) as dst:
             for i in range(1, src.count + 1):
                 reproject(
                     source=rasterio.band(src, i),
@@ -73,21 +81,24 @@ def prepare_mosaic_for_web(
                     src_crs=src.crs,
                     dst_transform=dst_transform,
                     dst_crs=target_crs,
-                    resampling=Resampling.bilinear
+                    resampling=Resampling.bilinear,
                 )
-            
+
             # Copy tags and color interpretation
             dst.update_tags(**src.tags())
             dst.colorinterp = src.colorinterp
-    
+
     if progress_callback:
         progress_callback(100, 100, "Mosaic prepared for web visualization")
-    
+
     return output_path
 
 
 def geotiff_to_web_tiles(
-    geotiff_path: str, output_dir: str, zoom_levels: Tuple[int, int] = (8, 15), use_gdal_raster: bool = False
+    geotiff_path: str,
+    output_dir: str,
+    zoom_levels: Tuple[int, int] = (8, 15),
+    use_gdal_raster: bool = False,
 ) -> str:
     """Convert GeoTIFF to web tiles for interactive display.
 
@@ -115,6 +126,7 @@ def geotiff_to_web_tiles(
 
     # Use gdal raster tile if explicitly requested and available
     if use_gdal_raster:
+
         def _has_gdal_raster_tile() -> bool:
             """Check if 'gdal raster tile' command is available."""
             try:
@@ -154,23 +166,25 @@ def geotiff_to_web_tiles(
             ]
 
             try:
-                print(f"Running gdal raster tile: {' '.join(cmd)}")
+                logger.info(f"Running gdal raster tile: {' '.join(cmd)}")
                 result = subprocess.run(cmd, check=True, capture_output=True, text=True)
                 if result.stdout:
-                    print("GDAL stdout:", result.stdout)
+                    logger.debug("GDAL stdout: %s", result.stdout)
                 if result.stderr:
-                    print("GDAL stderr:", result.stderr)
+                    logger.debug("GDAL stderr: %s", result.stderr)
                 return str(output_dir)
             except subprocess.CalledProcessError as e:
-                print(f"gdal raster tile failed (return code {e.returncode}):")
-                print(f"Command: {' '.join(cmd)}")
+                logger.error(f"gdal raster tile failed (return code {e.returncode}):")
+                logger.error(f"Command: {' '.join(cmd)}")
                 if e.stdout:
-                    print(f"Stdout: {e.stdout}")
+                    logger.error(f"Stdout: {e.stdout}")
                 if e.stderr:
-                    print(f"Stderr: {e.stderr}")
+                    logger.error(f"Stderr: {e.stderr}")
                 raise RuntimeError(f"gdal raster tile failed: {e}")
         else:
-            raise RuntimeError("gdal raster tile not available. Use default gdal2tiles or install gdal with raster tile support.")
+            raise RuntimeError(
+                "gdal raster tile not available. Use default gdal2tiles or install gdal with raster tile support."
+            )
 
     # Use traditional gdal2tiles.py (default)
     cmd = [
@@ -188,21 +202,23 @@ def geotiff_to_web_tiles(
     ]
 
     try:
-        print(f"Running gdal2tiles fallback: {' '.join(cmd)}")
+        logger.info(f"Running gdal2tiles fallback: {' '.join(cmd)}")
         result = subprocess.run(cmd, check=True, capture_output=True, text=True)
         if result.stdout:
-            print("gdal2tiles stdout:", result.stdout)
+            logger.debug("gdal2tiles stdout: %s", result.stdout)
         if result.stderr:
-            print("gdal2tiles stderr:", result.stderr)
+            logger.debug("gdal2tiles stderr: %s", result.stderr)
         return str(output_dir)
     except subprocess.CalledProcessError as e:
-        print(f"gdal2tiles failed (return code {e.returncode}):")
-        print(f"Command: {' '.join(cmd)}")
+        logger.error(f"gdal2tiles failed (return code {e.returncode}):")
+        logger.error(f"Command: {' '.join(cmd)}")
         if e.stdout:
-            print(f"Stdout: {e.stdout}")
+            logger.error(f"Stdout: {e.stdout}")
         if e.stderr:
-            print(f"Stderr: {e.stderr}")
-        raise RuntimeError(f"Tile generation failed with both gdal raster tile and gdal2tiles: {e}")
+            logger.error(f"Stderr: {e.stderr}")
+        raise RuntimeError(
+            f"Tile generation failed with both gdal raster tile and gdal2tiles: {e}"
+        )
     except FileNotFoundError:
         raise RuntimeError(
             "Neither 'gdal raster tile' nor 'gdal2tiles.py' found. Install GDAL tools."
@@ -276,7 +292,7 @@ def create_simple_web_viewer(
             boundary_geojson = json.dumps(boundary_geojson)
 
         except Exception as e:
-            print(f"Warning: Could not process region file {region_file}: {e}")
+            logger.warning(f"Could not process region file {region_file}: {e}")
             boundary_geojson = None
 
     html_content = f"""<!DOCTYPE html>
@@ -389,7 +405,7 @@ def create_coverage_summary_map(
         Path to created HTML file
     """
     from .visualization import analyze_geotiff_coverage
-    
+
     # Analyze coverage
     coverage = analyze_geotiff_coverage(geotiff_paths)
 
@@ -508,4 +524,4 @@ def create_coverage_summary_map(
     with open(output_html, "w") as f:
         f.write(html_content)
 
-    return output_html
\ No newline at end of file
+    return output_html
